/*Copyright and Author
 *
 *This program is free software, distributed under the terms of the GNU General Public License Version 2.
 *
 * \author Leena <leena_m@spanservices.com>, (SPAN Infotech India Pvt. Ltd., Bangalore)
 *
 * \module diameter
 * 
 * \created 24\03\09
 */

/*! \file
  \brief Diameter message avp related info.
  
 */

#ifndef _DIAMETER_AVP_H
#define _DIAMETER_AVP_H

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

#define VENDOR_CODE        22736

/*! \brief The various AVPs supported in the Diameter message */
enum avp_code_num {
	AVP_USER_NAME			  							=    1,
	AVP_SESSION_TIMEOUT               =   27,
	AVP_ACCT_INTERIM_INTERVAL	  			=   85,
	AVP_HOST_IP_ADDRESS               =  257,
 	AVP_AUTH_APPLICATION_ID           =  258,
	AVP_ACCT_APPLICATION_ID           =  259,
	AVP_VENDOR_SPECIFIC_APPLICATION_ID=  260,
 	AVP_REDIRECT_MAX_CACHE_TIME       =  262,
	AVP_SESSION_ID                    =  263,
	AVP_ORIGIN_HOST                   =  264,
	AVP_VENDOR_ID                     =  266,
 	AVP_RESULT_CODE                   =  268,
	AVP_PRODUCT_NAME                  =  269,
 	AVP_DISCONNECT_CAUSE              =  273,
	AVP_AUTH_REQUEST_TYPE             =  274,
 	AVP_AUTH_GRACE_PERIOD             =  276,
	AVP_AUTH_SESSION_STATE            =  277,
 	AVP_ORIGIN_STATE_ID               =  278,
	AVP_FAILED			  								=  279,
 	AVP_ERROR_MESSAGE                 =  281,
 	AVP_DESTINATION_REALM             =  283,
	AVP_ACCOUNTING_SUB_SESSION_ID     =  287,
  AVP_AUTHORIZATION_LIFETIME        =  291,	
	AVP_DESTINATION_HOST              =  293,
 	AVP_TERMINATION_CAUSE             =  295,
 	AVP_ORIGIN_REALM                  =  296,
	AVP_ACCT_RECORD_TYPE		  				=  480,
	AVP_ACCT_REALTIME_REQUIRED	  		=  483,
	AVP_ACCT_RECORD_NUMBER		  			=  485,
	/* \brief Avps which represent each member of the cdr structure */
	AVP_AST_ACCT_CODE 								=	 301,
	AVP_AST_SRC 											=  302,
 	AVP_AST_DST 											=  303,
 	AVP_AST_DST_CTX 									=  304,
 	AVP_AST_CLID 											=  305,
	AVP_AST_CHAN 											=  306,	
	AVP_AST_DST_CHAN 									=  307,
	AVP_AST_LAST_APP 									=  308,
  AVP_AST_LAST_DATA									=  309,
 	AVP_AST_START_TIME 								=  310,
	AVP_AST_ANSWER_TIME 							=  311,
 	AVP_AST_END_TIME 									=  312,
	AVP_AST_DURATION 									=  313,
 	AVP_AST_BILL_SEC 									=  314,
 	AVP_AST_DISPOSITION 							=  315,
	AVP_AST_AMA_FLAGS 								=  316,	
	AVP_AST_UNIQUE_ID 								=  317,
 	AVP_AST_USER_FIELD 								=  318,
	AVP_ACCT_SESSION_ID								=  319
};

/*! \brief The various Accounting Realtime AVP values */
enum avp_acct_realtime {
	DELIVER_AND_GRANT = 1,
	GRANT_AND_STORE 	= 2,
	GRANT_AND_LOSE 		= 3
};

/*brief The various result codes returned for the response messages sent */
enum diameter_msg_result_code {
  DIAMETER_SUCCESS 								 		= 2001,
  DIAMETER_APPLICATION_UNSUPPORTED 		= 3007,
  DIAMETER_UNKNOWN_PEER 					 		= 3010,
  DIAMETER_UNSUPPORTED_VERSION 				= 5011,
  DIAMETER_COMMAND_UNSUPPORTED 				= 3001,
  DIAMETER_UNABLE_TO_DELIVER 					= 3002,
  DIAMETER_REALM_NOT_SERVED 					= 3003,
  DIAMETER_INVALID_HDR_BITS 					= 3008,
  DIAMETER_INVALID_AVP_BITS 					= 3009,
  DIAMETER_TOO_BUSY 									= 3004,
  DIAMETER_LOOP_DETECTED 							= 3005,
  DIAMETER_OUT_OF_SPACE 							= 4002,
  DIAMETER_NO_COMMON_APPLICATION 			= 5010,
  DIAMETER_AVP_UNSUPPORTED 						= 5001,
  DIAMETER_UNKNOWN_SESSION_ID 				= 5002,
  DIAMETER_INVALID_AVP_VALUE 					= 5004,
  DIAMETER_MISSING_AVP 								= 5005,
  DIAMETER_RESOURCES_EXCEEDED 				= 5006,
  DIAMETER_CONTRADICTING_AVPS 				= 5007,
  DIAMETER_AVP_NOT_ALLOWED 						= 5008,
  DIAMETER_AVP_OCCURS_TOO_MANY_TIMES 	= 5009,
  DIAMETER_UNABLE_TO_COMPLY 					= 5012,
  DIAMETER_INVALID_BIT_IN_HEADER 			= 5013,
  DIAMETER_INVALID_AVP_LENGTH 				= 5014,
  DIAMETER_INVALID_MESSAGE_LENGTH 		= 5015,
  DIAMETER_INVALID_AVP_BIT_COMBO 			= 5016
};

/*! \brief The various Accounting record types */
enum avp_acct_record {
	AVP_ACCT_EVENT_RECORD 	= 1,
	AVP_ACCT_START_RECORD 	= 2,
	AVP_ACCT_INTERIM_RECORD = 3,
	AVP_ACCT_STOP_RECORD 		= 4
};

/*! \brief The varoius disconnect causes for a DPR message */
enum avp_disconnect_cause {
	REBOOTING 									= 0,
	BUSY 												= 1,
	DO_NOT_WANT_TO_TALK_TO_YOU 	= 2
};

/*! \brief The various datatypes of the AVP */
enum avp_datatype {
	AVP_TYPE_UNDEFINED = 0,
 	AVP_DATA_TYPE,
 	AVP_STRING_TYPE,
  AVP_ADDRESS_TYPE,
  AVP_INTEGER32_TYPE,
 	AVP_INTEGER64_TYPE,
 	AVP_TIME_TYPE,
	AVP_ENUMERATED_TYPE,
	AVP_UNSIGNED32_TYPE,
	AVP_UNSIGNED64_TYPE,
	AVP_UTF8STRING_TYPE,
	AVP_GROUPED_TYPE,
};

/*! \brief The flags of the AVP message */
enum avp_flag {
	AVP_FLAG_NONE               = 0x00,
  AVP_FLAG_MANDATORY          = 0x40,
  AVP_FLAG_RESERVED           = 0x1F,
 	AVP_FLAG_VENDOR_SPECIFIC    = 0x80,
	AVP_FLAG_END_TO_END_ENCRYPT = 0x20,
	AVP_FLAG_UNDEFINED,	    
};


/*! \brief Structure to hold the AVP data */
struct avp {
 	int 							length;			/*!< AVP length */
	unsigned int 			code;				/*!< AVP Code */	
	unsigned int 			vendor_id;	/*!< AVP Vendor-ID */
	enum avp_flag 		flag;				/*!< AVP Flags */
 	enum avp_datatype type;				/*!< AVP Data format */
 	char* 						data;				/*!< Data content of the AVP */
 	struct avp* 			next;				/*!< The next AVP to follow */
};


/*! 
 * \brief Build and add the built AVP to the AVP list of the Diameter message
 *
 * \param avp_code avp code of the AVP to be built
 * \param avp_data the data part of the AVP
 * \param avpdata_length the length of the AVP data
 * \param avp_list the list in which the built AVP is to be inserted
 * \param vendor_id vendor id of the avp
 * \param avpflag_value value of the avp flag which is to be set
 *
 * \retval 1 on success and 0 on failure 
 */
int build_and_add_avp(int avp_code, const char* avp_data,  unsigned int avpdata_length, struct avp** avp_list, unsigned int vendor_id, enum avp_flag avpflag_value);

/*!
 * \brief Builds and adds the required avps of a diameter message 
 *
 * \param msg_code message code of the diameter message whose avps are to be built
 * \param avplist list where the built avps are inserted
 *
 * \retval 1 on success and 0 on failure 
 */
int build_mandatory_avps(int msg_code, struct avp** avplist);

/*!
 * /brief Checks if Origin-Host avp and Origin-Realm avps are present in the avp list
 *
 * /param msgavp avplist in which the avps are looked for
 *
 * \retval 1 if the avps are present else 0 
 */
int check_host_nrealm_presence(struct avp** avp_list);

/*!
 * /brief Checks if a particular avp is present in an avplist 
 *
 * /param avp_list avp list in which the avp is looked for
 * /param avpcode avp code of the avp which is looked for
 *
 * \retval 1 if the avp is present else 0 
 */
int check_avp_presence(struct avp** avp_list, enum avp_code_num avpcode);

/*!
 * /brief Checks if the ACCT_REALTIME_REQUIRED avp's value is of a particular value in an avp list
 *
 * /param avp_p avp list in which the ACCT_REALTIME_REQUIRED is searched for
 * /param avp_val avp value against which ACCT_REALTIME_REQUIRED avp's values is compared
 *
 * \retval 1 if the avp values match else 0 
 */
int realtime_avpcheck(struct avp* avp_p, enum avp_acct_realtime avp_val);

/*!
 * /brief Gets the value of an avp from an avp list
 *
 * /param avplist avp list in which the avp is searched
 * /param avpcode avp code of the avp which is searched
 *
 * \return avp value if the avp is present else NULL
 */
char* get_avpval(struct avp** avplist, enum avp_code_num avpcode);

/*!
 /brief Deletes an avp list
 *
 * /param avp_list avp list which is removed 
 */
void avplist_cleanup(struct avp* avp_list);

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif


#endif /* _DIAMETER_AVP_H */
