/*Copyright and Author
 *
 *This program is free software, distributed under the terms of the GNU General Public License Version 2.
 *
 * \author Leena <leena_m@spanservices.com>, (SPAN Infotech India Pvt. Ltd., Bangalore)
 *
 * \module diameter
 * 
 * \created 01\04\09
 */
  
/*! \file
  \brief About diameter client(peer) events session info.
  
 */

#ifndef _DIAMETER_SESSION_H
#define _DIAMETER_SESSION_H

#include "diameter_message.h"

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

/*! \brief state definitions of a Diameter Client */
enum peer_state_t {
	CLOSED               = 0,    /*!< Not connected */
	WAIT_CONN_ACK        = 1,    /*!< Connecting - waiting for Ack */
	WAIT_I_CEA           = 3,    /*!< Connecting - waiting for Capabilities Exchange Answer */
	WAIT_RETURNS         = 4,		 /*!< When waiting for some event is over */
	I_OPEN               = 5,    /*!< Connected as initiator */
  R_OPEN               = 6,    /*!< Connected as responder */
 	CLOSING              = 7,    /*!< Closing the connection */
	ELECT								 = 8,    /*!< Election to be held */
	WAIT_CONN_ACK_ELECT	 = 9		 /*!< Waiting for Ack and election held */
};

/*! \brief Diameter peer's accounting states */
enum acct_state_t {
	IDLE				=	10,		/*!< When nothing is done */
	OPEN				=	11,		/*!< On receiving successful answers or when a new Accounting record is sent */
	PENDING_S		=	12,		/*!< Waiting for answer after Accounting START record is sent */
	PENDING_I		=	13,		/*!< Waiting for answer after Accounting INTERIM record is sent */
	PENDING_L		=	14, 	/*!< Waiting for answer after Accounting STOP record is sent */
	PENDING_E		=	15,  	/*!< Waiting for answer after Accounting EVENT record is sent */
	PENDING_B		=	16		/*!< When stored accounting records are sent */
};

/*! \brief Diameter peer's accounting events */
/*!	\note Here "realtime" refers to the value of Accounting-Realtime-Required AVP */
enum acct_event_t {
	ACCT_REQUEST_ACCESS 													= 200, 	/*!< Client or device requests access */
	ACCT_ONE_TIME_SERVICE 												= 201, 	/*!< When accountimg EVENT record is to be sent */
	ACCT_RECORDS_IN_STORAGE 											= 202,
	ACCT_SUCCESSFUL_START_ANS_RECVD 							= 203, 	/*!< Successful accounting START answer received */
	ACCT_FAILURE_TOSEND_BA_NOT_DELIVER_AND_GRANT 	= 204, 	/*!< Failure to send and buffer space available and realtime equal to GRANT_AND_LOSE */
	ACCT_FAILURE_TOSEND_NBA_GRANT_NLOSE 					= 205, 	/*!< Failure to send and no buffer space available and realtime equal to GRANT_AND_LOSE */
	ACCT_FAILURE_TOSEND_NBA_NOT_GRANT_NLOSE 			= 206,	/*!< Failure to send and no buffer space available and realtime NOT equal to GRANT_AND_LOSE */
	ACCT_FAILED_START_ANS_RECVD 									= 207, 	/* Failed accounting START answer recived */
	ACCT_FAILED_ANSWER_NOT_GRANT_NLOSE 						= 208, 	/* Failed accounting answer recieved but realtime NOT equal to GRANT_AND_LOSE */
	ACCT_USR_SERVICE_TERMINATED 									= 209, 	/*!< User service terminated */
	ACCT_INTERIM_INTERVAL_ELAPSES 								= 210, 	/* When accounting INTERIM interval elapses */
	ACCT_SUCCESSFUL_INTERIM_ANS_RECVD 						= 211, 	/*!< Successful accounting INTERIM answer received */
	ACCT_SUCCESSFUL_EVENT_ANS_RECVD 							= 212, 	/*!< Successful accounting INTERIM record answer received */
	ACCT_FAILURE_TOSEND_BA 												= 213, 	/* Failure to send and buffer space available */
	ACCT_FAILURE_TOSEND_NBA 											= 214, 	/* Failure to send and no buffer space available */
	ACCT_FAILED_EVENT_ANS_RECVD 									= 215, 	/* Failed accounting EVENT answer received */
	ACCT_SUCCESSFUL_ACCT_ANS_RCVD 								= 216, 	/*!< Successful accounting START answer received */
	ACCT_FAILURE_TOSEND 													= 217, 	/* Failure to send */
	ACCT_FAILED_ACCT_ANS_RCVD 										= 218, 	/* Failed accounting answer received */
	ACCT_SUCCESSFUL_STOP_ANS_RECVD 								= 219, 	/*!< Successful accounting STOP answer received */
	ACCT_FAILED_STOP_ANS_RECVD										= 220,	/*!< Failed accounting stop record answer received */
	ACCT_FAILED_INTERIM_ANS_RCVD_GRANT_NLOSE 			= 221, 	/* Failed INTERIM answer received and realtime equal to GRANT_AND_LOSE */
	ACCT_FAILED_INTERIM_ANS_RCVD_NOT_GRANT_NLOSE 	= 222 	/* Failed INTERIM answer received and realtime equal NOT to GRANT_AND_LOSE */
};

/*! \brief Peer events definition */
enum peer_event_t {
	START                   = 101,  	/*!< Start connection attempt */
	STOP                    = 102,  	/*!< Stop */
	TIMEOUT                 = 103,  	/*!< Time-out */
 	WIN_ELECTION    				= 104,  	/*!< Winning the election */
	I_RCV_CONN_ACK  				= 106,  	/*!< Initiator - Received connection Ack */
	I_RCV_CONN_NACK 				= 107,  	/*!< Initiator - Received connection NAck */
	I_RCV_CER               = 108,  	/*!< Initiator - Receiver Capabilities Exchange Request */
	R_RCV_CER								= 109,		/*!< Receiver - Receiver Capabilities Exchange Request */
 	I_RCV_CEA               = 110, 	  /*!< Initiator - Receiver Capabilities Exchange Answer */
	R_RCV_CEA               = 111,  	/*!< Receiver - Receiver Capabilities Exchange Answer */
 	I_RCV_NON_CEA   				= 112,  	/*!< Initiator - Received non-Capabilities Exchange Answer */
  I_RCV_DPR               = 113,  	/*!< Initiator - Received Disconnect Peer Request */
 	R_RCV_DPR               = 114,  	/*!< Receiver - Received Disconnect Peer Request */
	I_RCV_DPA               = 115,  	/*!< Initiator - Received Disconnect Peer Answer */
 	R_RCV_DPA               = 116,  	/*!< Received - Received Disconnect Peer Answer */
  I_RCV_DWR               = 117,  	/*!< Initiator - Received Diameter Watch-dog Request */
 	R_RCV_DWR               = 118,  	/*!< Receiver - Received Diameter Watch-dog Request */
 	I_RCV_DWA               = 119,  	/*!< Initiator - Received Diameter Watch-dog Answer */
 	R_RCV_DWA               = 120,  	/*!< Receiver - Received Diameter Watch-dog Answer */
	R_CONN_CER							= 121,    /*!< Receiver - An acknowledgement is received stating that the transport	connection has been established,and the associated CER has arrived */
 	SEND_MESSAGE    				= 122,  	/*!< Send a message */
 	I_RCV_MESSAGE   				= 123,  	/*!< Initiator - Received a message other than CER, CEA, DPR, DPA, DWR or DWA */
  R_RCV_MESSAGE   				= 124,  	/*!< Receiver - Received a message other than CER, CEA, DPR, DPA, DWR or DWA */
	I_PEER_DISC             = 125,  	/*!< Initiator - Peer disconnected */
  R_PEER_DISC             = 126,  	/*!< Responder - Peer disconnected */
};

/*! \brief Peer actions definition */
enum action_t {
	SND_CONN_REQ = 20,	/*!< A transport connection is initiated with the peer */
	ACCEPT,							/*!< The incoming connection associated with the R-Conn-CER is accepted as the responder connection */
	PROCESS_CER,				/*!< The CER associated with the R-Conn-CER is processed */
	SND_CER,        		/*!< A CER message is sent to the peer */
	CLEANUP,        		/*!< The connection is shutdown, and any local resources are freed */
	ERROR,							/*!< The transport layer connection is disconnected,in response to an error condition */
	PROCESS_CEA,    		/*!< A received CEA is processed */
	SND_DPR,						/*!< A DPR message is sent to the peer */
	SND_DPA,						/*!< A DPA message is sent to the peer */
	I_DISC,							/*!< The transport layer initiator connection is disconnected */
	R_DISC,							/*!< The transport layer receiver connection is disconnected */
	SND_MESSAGE,				/*!< A message is  sent */
	SND_DWR,						/*!< A DWR message is sent */
	SND_DWA,						/*!< A DWA message is sent */
	PROCESS_DWR,				/*!< The DWR message is serviced */
	PROCESS_DWA,				/*!< The DWA message is serviced */
	PROCESS,						/*!< A message is serviced */
	REJECT							/*!< An incoming connection is rejected */
};

/*! \brief Structure to hold the accounting session information */
struct acct_session {
	int 									interim_time_val;       /*!< The interim interval period */
 	int 									interim_val;            /*!< Value of the current interim record */
 	int 									acct_timer_id;          /*!< Timer id of the accounting interval timer */
 	long int 							answer_time;       	    /*!< Time at which the call was answered */
	unsigned int 					session_id;             /*!< Session id of the message */
	unsigned int 					acct_subsession_id;     /*!< The accounting sub-session's id */
 	ast_mutex_t 					mutex;                  /*!< Mutex for the session */
	enum acct_state_t 		state;                	/*!< Current state of the accounting sub-session */
 	struct avp* 					avp_toadd;              /*!< Asterisk cdr info Avps to be appended in the message */
 	struct transaction_t* trans;            			/*!< Transactions of the accounting messages */
};

/*! 
 * \brief Builds and sends responses for the error requests received and also handles the received error responses.
 *
 * \param error_code error message code of the error request received or message code of the error response received 
 * \param err_msg the error request or error response received 
 *
 * \retval 1 on success and 0 on failure
 */
int handle_error_message(int error_code, struct message *err_msg);

/*!
 * \brief Creates a session for an accounting request of a particular record.
 *
 * \param avplist list of avps which contain cdr information of the call for whichthe accounting information is sent
 * \param acct_rec_type record type of the accounting request to be sent
 *
 * \retval 1 on success and 0 on failure
 */
int send_accounting_request(struct avp* avplist, enum avp_acct_record acct_rec_type);
/*!
 * \brief Implements an action triggered by an event.
 *
 *\param action action to take place
 *\param msg message received with the event which triggered the action else NULL
 *                                                                                                                           
 *\retval 1 on success and 0 on failure
 */
int take_action(enum action_t action, struct message *msg);

/*!
 * \brief Updates the state of the diameter client.
 *
 * \param peer_state state to which the diameter client is updated to
 */
void update_diameterbase_state(enum peer_state_t peer_state);
/*! 
 * \brief Updates the state of the diameter client and takes the next corresponding action. 
 *
 * \param event the event that took place 
 * \param msg the message that reflects the event if present else NULL
 *
 * \retval 1 on success and 0 on failure
 */
int process_event(enum peer_event_t peer_event, struct message *msg);

/*!
 * \brief Compare the entries in the hash table.
 *
 * \param session_a,session_b entried to be compared 
 *
 * \retval 0 on match
 */
int compare_session(const struct acct_session* session_a, const struct acct_session* session_b);

/*!
 * \brief Hash function for the hash table used for sessions.
 *
 * \param session element which is searched in hash table
 *
 * \retval returns the session id of the session
 */
unsigned int get_session_hash(const struct acct_session* session);

/*!
 * \brief Gets the cdr info avplist of an accounting session.
 * 
 * \param session accounting session whose avplist is returned
 *
 * returns the avplist on success else NULL 
 */
struct avp* get_session_avp(struct acct_session* session);

/*!\brief Clones the cdr info avplist of an accounting session.
 *
 * \param session accounting session whose avplist is cloned
 *
 * \returns the cloned avplist on succesd else NULL
 */
struct avp* get_session_avp_clone(struct acct_session* session);

/*!
 * \brief Gets the interim interval of an accounting session.
 * \param session accounting session whose interim interval is returned
 *
 * \retval interim interval on success else 0
 */
int get_session_interim_val(struct acct_session* session);

/*!
 * \brief Gets the answertime(time at which call was answered) of an accounting session.
 * \param session accounting session whose answertime is returned
 *
 * \retval answertime on success else 0
 */
long int get_session_answertime(struct acct_session* session);	

/*!
 * \brief Frees and deletes an accounting session,
 *
 * \param pvdata accounting session which is cleaned.
 */
void cleanup_acct_session(void* pvdata);

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif

#endif /*_DIAMETER_SESSION_H */
