/*Copyright and Author
 *
 *This program is free software, distributed under the terms of the GNU General Public License Version 2.
 *
 * \author Leena <leena_m@spanservices.com>, (SPAN Infotech India Pvt. Ltd., Bangalore)
 *
 * \module diameter
 * 
 * \created 09\04\09
 */

/*! \file
  \brief Transaction related info.
  
 */

#ifndef _DIAMETER_TRANSACTION_H
#define _DIAMETER_TRANSACTION_H

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

/*! Max. no. of retransmissions allowed for a message */
#define MAX_RETRANSMISSIONS 3

/*! \brief Structure to hold the  Diameter transaction information */
struct transaction_t {
	unsigned int 					endtoend_identifier;      /*!< End-to-end id of the message */
 	unsigned int 					hopbyhop_identifier;      /*!< Hop-by-hop id of the message */
 	int 									retransmissions;          /*!< No.of retransmissions of the request message */
	int 									timer_id;                 /*!< Timer id of 30 seconds timer */
 	struct message*				request_message;       		/*!< Request message */
 	struct transaction_t*	next;             				/*!< The next transaction in the transaction list */
};

/*! 
 * \brief Creates a transaction and add it intoa list.
 *
 * \param diam_message the message for which the transaction is created.
 * \param pvdata the session of the diameter message if is an accounting message else NULL
 *
 * \returns the created transaction on success else NULL
 */
struct transaction_t* transaction_init(struct message* diam_message, void* pvdata);

/*!
 * /brief Finds a message's transaction in a transaction list.
 *
 * /param msg message whose transaction is searched
 * /param translist the list in which the transaction is searched
 *
 * /retval 0 if transaction found else 0
 */
int find_transaction(struct message *msg, struct transaction_t* translist);

/*!
 * /brief Frees and deletes a transaction list.
 *
 * /param trans_t transaction list which is cleaned up.
 */
void transaction_cleanup(struct transaction_t** trans_t);

/*!
 * /brief Frees a messages's transaction from a list.
 *
 * /param message whose transaction is to be freed
 * /param trans the transaction list from which the message is freed.
 *
 * /retval 1 on success else 0
 */
int free_transaction(struct message *message, struct transaction_t** trans);

/*!
 * /brief Sends a diameter message and starts a timer if the message is a request.
 * 
 * /param message_to_send message that is to be sent
 * /param trans transaction of the message to be sent
 * 
 * /retval 1 on sucess and 0 on failure
 */
int transaction_send_message(struct message *message_to_send, struct transaction_t* trans);

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif

#endif  /*_DIAMETER_TRANSACTION_H */

